% Author:
% Date: 22.04.2008

?- consult('Summe.pl').
?- consult('Vereinfachen.pl').
?- consult('Auswerten.pl').
?- consult('Produkt.Pl').

% ableiten = ableitung und vereinfachen
ableiten(E, X, A) :- ableitung(E, X, A1), vereinfachen(A1, A2), auswerten(A2, A3), vereinfachen(A3, A).

% ableitung(+Term, +Ableitungsvariable, -AbgeleiteterTerm)

% Konstantenregel
ableitung(K, X, 0) :- atomic(K), K \== X, !.

% Ableitung von x
ableitung(X, X, 1) :- !.

% Vorzeichenregel
ableitung(-T, X, -Ta) :-  !, ableitung(T, X, Ta).

% Summenregel
ableitung(T1 + T2, X, T1a + T2a) :- !,
             ableitung(T1, X, T1a), ableitung(T2, X, T2a).
             
% Differenzenregel
ableitung(T1 - T2, X, T1a-T2a) :- !,
             ableitung(T1, X, T1a), ableitung(T2, X, T2a).
             
% Faktorregel
ableitung(Konst*T, X, Konst*Ta) :-
                   atomic(Konst), Konst \== X, !, ableitung(T, X, Ta).
                   
% Produktregel
ableitung(T1*T2, X, T1*T2a + T2*T1a) :- !,
                 ableitung(T1, X, T1a), ableitung(T2, X, T2a).
                 
% Quotientenregel
ableitung(T1/T2, X, (T1*T2a - T2* T1a)/(T2*T2)) :- !,
                 ableitung(T1, X, T1a), ableitung(T2, X, T2a).

% Ableitung der Grundfunktionen
ableitung(X^N, X, N*X^(N - 1)) :- atomic(N), N \== X, !.
ableitung(sin(X), X, cos(X)) :- !.
ableitung(cos(X), X, -sin(X)) :- !.
ableitung(ln(X), X, 1/X) :- !.
ableitung(e(X), X, e(X)) :- !.

% Kettenregel
ableitung(T, X, Ta*Ga) :- T =.. [_,G], G \== X, !, ableitung(T, G, Ta), ableitung(G, X, Ga).

% Allgemeine Ableitung
ableitung(T, X, Ta) :- T =.. [F,X], atom_concat(F, 'S', G), Ta =.. [G,X].
